#!/usr/bin/env python3
import argparse
from collections import defaultdict
import jinja2
import sys
import re
sys.path.append('..')

from squad_client.core.api import SquadApi
from squad_client.core.models import Squad, SquadObject, ALL

parser = argparse.ArgumentParser()
parser.add_argument('--debug',
        action='store_true',
        dest='debug',
        help='display debug messages')
parser.add_argument('--group',
        dest='group',
        required=True,
        help='SQUAD group slug')
parser.add_argument('--project',
        dest='project',
        required=True,
        help='SQUAD project slug')
parser.add_argument('--build',
        dest='build',
        required=True,
        help='Build ID within a project')
parser.add_argument('--token',
        dest='token',
        required=True,
        help='SQUAD authorization token')
parser.add_argument('--template',
        dest='template',
        required=True,
        help='Rendering template')
parser.add_argument('--intro',
        dest='intro',
        required=False,
        default=None,
        help='Free form text or HTML to be added to template introduction')
parser.add_argument('--output',
        dest='output',
        required=True,
        help='Output filename')
parser.add_argument('--squad-host',
        dest='server',
        default='https://qa-reports.linaro.org/',
        help='SQUAD server')

args = parser.parse_args()

SquadApi.configure(url=args.server, token=args.token)

def getid(url):
    matches = re.search('^.*/(\d+)/$', url)
    try:
        _id = int(matches.group(1))
        return _id
    except ValueError:
        print('Could not get id for %s' % url)
        return -1

group = Squad().group(args.group)
project = group.project(args.project)
environments = project.environments(count=ALL)
suites = project.suites(count=ALL)
build = project.build(args.build)
testruns = build.testruns(fields='id,environment')
tests = build.tests(fields='id,short_name,status,environment,suite,test_run').values()
metrics = Squad().metrics(fields='id,short_name,result,test_run,suite', count=ALL, test_run__build=build.id).values()

env_summaries = {e.slug: {'pass': 0, 'fail': 0, 'xfail': 0, 'skip': 0} for e in environments.values()}
overall_summary = {'pass': 0, 'fail': 0, 'xfail': 0, 'skip': 0}
table = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
for test in sorted(tests, key=lambda obj: obj.short_name):
    env = environments[getid(test.environment)].slug
    suite = suites[getid(test.suite)].slug
    table[env]['tests'][suite].append(test)

    env_summaries[env][test.status] += 1
    overall_summary[test.status] += 1

# Sort suites
for env in table.keys():
    __suites = table[env]['tests']
    table[env]['tests'] = {s: __suites[s] for s in sorted(__suites)}



# TODO: add build and env to metrics table as well
for metric in metrics:
    testrun = testruns[getid(metric.test_run)]
    env = environments[getid(testrun.environment)].slug
    suite = suites[getid(metric.suite)].slug

    table[env]['metrics'][suite].append(metric)

intro = None
if args.intro:
    with open(args.intro, 'r') as f:
        intro = f.read()

templateLoader = jinja2.FileSystemLoader(searchpath="./")
templateEnv = jinja2.Environment(loader=templateLoader, trim_blocks=True, lstrip_blocks=True)
templateEnv.filters['getid'] = getid
template = templateEnv.get_template(args.template)
outputText = template.render(group=group, project=project, build=build,
                             intro=intro, server=args.server,
                             env_summaries=env_summaries, table=table)
with open(args.output, 'w') as reportFile:
    reportFile.write(outputText)



import jinja2
import uuid

from os import path
from io import StringIO

from squad_client import logging
from squad_client.core.api import SquadApi
from squad_client.core.models import SquadObject, Squad
from squad_client.exceptions import InvalidReportOutput, InvalidReportTemplate


logger = logging.getLogger(__name__)


class Report:
    def __init__(self, template, name=None, output=None, context=None):
        self.name = name if name else str(uuid.uuid1())
        self.template = template
        self.output = output
        self.context = context

    def generate(self):
        logger.info('Generating report "%s"' % self.name)
        values = self.context.fill() if self.context else {}
        template = self.get_template()
        return template.render(**values)

    def get_template(self):
        if self.template.endswith('.jinja2'):
            if path.isfile(self.template):
                abspath = path.abspath(self.template)
                loader = jinja2.FileSystemLoader(searchpath=path.dirname(abspath))
                environment = jinja2.Environment(loader=loader)
                template = environment.get_template(self.template)
            else:
                raise InvalidReportTemplate('Template file "%s" not found!' % self.template)
        else:
            template = jinja2.Template(self.template)
        return template


class ReportContext:
    class Context:
        pass

    def __init__(self, context={}):
        """
            Context dictionary should be in the format
            context = {
                'var1': {
                    'type': 'Build', # or any other available subclass of SquadObject
                    'filters': {
                        'param1': 'val1',
                        ...
                    }
                },
                ...
            }
        """
        self.context = []
        for name in context.keys():
            c = ReportContext.Context()
            if not name.isidentifier():
                name = name.replace(' ', '_')
            c.name = name
            c.type = context[name]['type']
            c.filters = context[name].get('filters') or {}
            self.context.append(c)

    def fill(self):
        logger.debug('Building report context')
        self.values = {}
        squad = Squad()
        for c in self.context:
            _type = SquadObject.get_type(c.type)
            self.values[c.name] = squad.fetch(_type, **c.filters)
        return self.values


class ReportGenerator:
    def __init__(self, squad_url, token=None):
        self.squad_url = squad_url
        self.token = token
        self.reports = []

    def add_report(self, name, template, output=None, context={}):
        report = Report(template, name, output, ReportContext(context))
        self.reports.append(report)
        return report

    def generate(self):
        for report in self.reports:
            SquadApi.configure(self.squad_url, self.token)
            output = report.generate()
            if report.output is None:
                # print to stdout
                print(output)
            elif type(report.output) is StringIO:
                report.output.write(output)
            elif type(report.output) is str and path.isfile(report.output):
                with open(report.output, 'w') as f:
                    f.write(output)
            else:
                raise InvalidReportOutput('Report "%s" output configuration is neither None, a file or StringIO object' % report.name)
        return self.reports



# This file is supposed to run in a squad instance for squad-client tests
#
# Some guidance to maintain this file:
#  - try to keep all values here
#  - do not delete any of the data anywhere (if deletion tests are needed, create one for the specific test)
#

from squad.core import models as m
from squad.core.tasks import RecordTestRunStatus
from squad.ci import models as mci

from rest_framework.authtoken.models import Token

user = m.User.objects.create(username='admin_user', is_superuser=True)
token = Token.objects.create(user=user, key='193cd8bb41ab9217714515954e8724f651ef8601')

group = m.Group.objects.create(slug='my_group')
group2 = m.Group.objects.create(slug='my_other_group')

project = group.projects.create(slug='my_project')
project_private = group.projects.create(slug='my_private_project', is_public=False)

build = project.builds.create(version='my_build')
build2 = project.builds.create(version='my_build2')
build3 = project.builds.create(version='my_build3')
build4 = project.builds.create(version='my_build4')
build5 = project.builds.create(version='my_build5')
build6 = project.builds.create(version='my_build6')

environment = project.environments.create(slug='my_env')
environment2 = project.environments.create(slug='my_other_env')
suite = project.suites.create(slug='my_suite')
suite2 = project.suites.create(slug='my_other_suite')

metadata_my_passed_test, _ = m.SuiteMetadata.objects.get_or_create(kind='test', suite=suite.slug, name='my_passed_test')
metadata_my_failed_test, _ = m.SuiteMetadata.objects.get_or_create(kind='test', suite=suite.slug, name='my_failed_test')
metadata_my_xfailed_test, _ = m.SuiteMetadata.objects.get_or_create(kind='test', suite=suite.slug, name='my_xfailed_test')
metadata_my_skipped_test, _ = m.SuiteMetadata.objects.get_or_create(kind='test', suite=suite.slug, name='my_skipped_test')

testrun = build.test_runs.create(environment=environment)
passed_test = testrun.tests.create(suite=suite, result=True, metadata=metadata_my_passed_test, build=testrun.build, environment=testrun.environment)
failed_test = testrun.tests.create(suite=suite, result=False, metadata=metadata_my_failed_test, build=testrun.build, environment=testrun.environment)
xfailed_test = testrun.tests.create(suite=suite, result=True, metadata=metadata_my_xfailed_test, has_known_issues=True, build=testrun.build, environment=testrun.environment)
skipped_test = testrun.tests.create(suite=suite, result=None, metadata=metadata_my_skipped_test, build=testrun.build, environment=testrun.environment)

RecordTestRunStatus()(testrun)

backend = mci.Backend.objects.create()
testjob = testrun.test_jobs.create(backend=backend, target=project, target_build=build)

emailtemplate = m.EmailTemplate.objects.create(name='my_emailtemplate')
suitemetadata = m.SuiteMetadata.objects.create(name='my_suitemetadata')
metricthreshold = m.MetricThreshold.objects.create(environment=environment, value=42)
report = build.delayed_reports.create()



import yaml

from os import path, chdir

from squad_client import logging
from squad_client.core.command import SquadClientCommand
from squad_client.report import ReportGenerator


logger = logging.getLogger(__name__)


class ReportCommand(SquadClientCommand):
    command = 'report'
    help_text = 'generate reports from yaml descriptor file'

    def register(self, subparser):
        parser = super(ReportCommand, self).register(subparser)
        parser.add_argument('--report-config', help='yaml file configuring the report. Assume `report.yml` by default', default='report.yml', dest='report_config')

    def run(self, args):
        logger.info('Running report')
        if not path.isfile(args.report_config):
            logger.error('Report config file "%s" not found!' % args.report_config)
            return False

        if args.report_config != 'report.yml':
            report_path = path.abspath(args.report_config)
            chdir(path.dirname(report_path))
            args.report_config = path.basename(report_path)

        return self.__generate__(args.report_config)

    def __generate__(self, report_config):
        logger.info('generating based on %s' % report_config)
        config = self.__parse_config__(report_config)
        generator = ReportGenerator(config.get('squad_url'), config.get('token'))
        for r in config.get('reports'):
            generator.add_report(r.get('name'), r.get('template'), output=r.get('output'), context=r.get('context'))

        generated = generator.generate()
        return len(generated) > 0

    def __parse_config__(self, report_config):
        with open(report_config) as f:
            config = yaml.load(f, Loader=yaml.FullLoader)
        return config



import subprocess as sp
import os

from .squad_service import SquadService


def run(coverage=False, tests=['discover'], verbose=False):
    squad_service = SquadService()
    if not squad_service.start() or not squad_service.apply_fixtures('tests/fixtures.py'):
        print('Aborting tests!')
        return False

    argv = ['-m', 'unittest'] + tests

    if len(tests) == 0:
        argv += ['discover']

    if verbose:
        argv += ['-v']

    if coverage:
        print('\t --coverage is enabled, run `coverage report -m` to view coverage report')
        argv = ['coverage', 'run', '--source', 'squad_client'] + argv
    else:
        argv = ['python3'] + argv

    env = os.environ.copy()
    env['LOG_LEVEL'] = 'ERROR'
    proc = sp.Popen(argv, env=env)
    proc.wait()

    squad_service.stop()
    return proc.returncode == 0



import unittest
from unittest.mock import patch
from io import StringIO


from squad_client.report import Report, ReportContext, ReportGenerator
from squad_client.exceptions import InvalidSquadObject


class ReportContextTest(unittest.TestCase):
    def test_basics(self):
        context = {
            'var1': {
                'type': 'Build',
                'filters': {
                    'param1': 'val1'
                }
            }
        }

        report_context = ReportContext(context)
        self.assertEqual(1, len(report_context.context))

        c = report_context.context[0]
        self.assertEqual('var1', c.name)
        self.assertEqual('Build', c.type)
        self.assertEqual({'param1': 'val1'}, c.filters)

    def test_invalid_object_type(self):
        context = {
            'var1': {
                'type': 'InvalidType',
            }
        }
        report_context = ReportContext(context)
        with self.assertRaises(InvalidSquadObject):
            report_context.fill()


class ReportTest(unittest.TestCase):

    def test_basic_report_generation(self):
        template = 'This is the most basic template'
        report = Report(template)
        generated = report.generate()
        self.assertEqual(template, generated)

    @patch('squad_client.core.models.Squad.fetch')
    def test_basic_report_generation_with_context(self, squad_fetch):
        squad_fetch.return_value = 'fetched string'
        template = 'Report: {{ dummy }}'
        context = ReportContext({
            'dummy': {
                'type': 'Test',
                'filters': {
                    'param1': 'val1'
                }
            }
        })
        report = Report(template, context=context)
        generated = report.generate()
        self.assertEqual('Report: fetched string', generated)


class ReportGeneratorTest(unittest.TestCase):

    @patch('squad_client.core.models.Squad.fetch')
    def test_basics(self, squad_fetch):
        squad_fetch.return_value = 'fetched string'
        template = 'Report: {{ dummy }}'
        output = StringIO()
        context = {
            'dummy': {
                'type': 'Test',
                'filters': {
                    'param1': 'val1'
                }
            }
        }
        generator = ReportGenerator('http://example.com')
        generator.add_report('dummy report', template, output=output, context=context)
        reports = generator.generate()

        self.assertEqual(1, len(reports))

        self.assertEqual('Report: fetched string', output.getvalue())



import unittest
import subprocess as sp
import os

from . import settings
from squad_client.core.api import SquadApi
from squad_client.core.models import Squad
from squad_client.utils import first

import squad_client.commands.submit_tuxbuild


class SubmitTuxbuildCommandTest(unittest.TestCase):

    testing_server = "http://localhost:%s" % settings.DEFAULT_SQUAD_PORT
    testing_token = "193cd8bb41ab9217714515954e8724f651ef8601"

    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(url=self.testing_server, token=self.testing_token)

    def submit_tuxbuild(self, tuxbuild):
        argv = [
            "./manage.py",
            "--squad-host",
            self.testing_server,
            "--squad-token",
            self.testing_token,
            "submit-tuxbuild",
            "--group",
            "my_group",
            "--project",
            "my_project",
            tuxbuild,
        ]

        env = os.environ.copy()
        env['LOG_LEVEL'] = 'INFO'
        proc = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE, env=env)
        proc.ok = False

        try:
            out, err = proc.communicate()
            proc.ok = proc.returncode == 0
        except sp.TimeoutExpired:
            self.logger.error(
                'Running "%s" time out after %i seconds!' % " ".join(argv)
            )
            proc.kill()
            out, err = proc.communicate()

        proc.out = out.decode("utf-8")
        proc.err = err.decode("utf-8")
        return proc

    def test_submit_tuxbuild_build(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/build.json")
        self.assertTrue(proc.ok, msg=proc.err)
        self.assertTrue(proc.err.count("Submitting 1 tests") == 3)

        build = (
            self.squad.group("my_group").project("my_project").build("next-20201021")
        )
        self.assertIsNotNone(build)
        self.assertEqual(
            sorted(squad_client.commands.submit_tuxbuild.ALLOWED_METADATA + ["id"]),
            sorted(list(build.metadata.__dict__.keys())),
        )
        self.assertIsNone(build.metadata.git_branch)

        build = (
            self.squad.group("my_group").project("my_project").build("v4.4.4")
        )
        self.assertIsNotNone(build)
        self.assertEqual(
            sorted(squad_client.commands.submit_tuxbuild.ALLOWED_METADATA + ["id"]),
            sorted(list(build.metadata.__dict__.keys())),
        )
        self.assertIsNone(build.metadata.git_branch)

        for arch in ["arm64", "x86"]:
            environment = (
                self.squad.group("my_group").project("my_project").environment(arch)
            )
            self.assertIsNotNone(environment, "environment %s does not exist" % (arch))

        suite = self.squad.group("my_group").project("my_project").suite("build")
        self.assertIsNotNone(suite)

        test = first(self.squad.tests(name="gcc-9-defconfig-b9979cfa"))
        self.assertEqual("build/gcc-9-defconfig-b9979cfa", test.name)
        self.assertEqual("pass", test.status)

        test = first(self.squad.tests(name="gcc-9-defconfig-5b09568e"))
        self.assertEqual("build/gcc-9-defconfig-5b09568e", test.name)
        self.assertEqual("fail", test.status)

    def test_submit_tuxbuild_buildset(self):
        os.environ["KERNEL_BRANCH"] = "master"
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/buildset.json")
        self.assertTrue(proc.ok, msg=proc.out)
        self.assertIn("Submitting 3 tests", proc.err)

        build = (
            self.squad.group("my_group").project("my_project").build("next-20201030")
        )
        self.assertIsNotNone(build)
        self.assertEqual(
            sorted(squad_client.commands.submit_tuxbuild.ALLOWED_METADATA + ["id"]),
            sorted(list(build.metadata.__dict__.keys())),
        )
        self.assertEqual(build.metadata.git_branch, os.environ.get("KERNEL_BRANCH"))

        environment = (
            self.squad.group("my_group").project("my_project").environment("x86")
        )
        self.assertIsNotNone(environment)

        suite = self.squad.group("my_group").project("my_project").suite("build")
        self.assertIsNotNone(suite)

        test = first(self.squad.tests(name="gcc-8-allnoconfig"))
        self.assertEqual("build/gcc-8-allnoconfig", test.name)
        self.assertEqual("pass", test.status)

        test = first(self.squad.tests(name="gcc-8-tinyconfig"))
        self.assertEqual("build/gcc-8-tinyconfig", test.name)
        self.assertEqual("pass", test.status)

        test = first(self.squad.tests(name="gcc-8-x86_64_defconfig"))
        self.assertEqual("build/gcc-8-x86_64_defconfig", test.name)
        self.assertEqual("pass", test.status)

    def test_submit_tuxbuild_empty(self):
        proc = self.submit_tuxbuild("")
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn("No such file or directory: ''", proc.err)

    def test_submit_tuxbuild_malformed(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/malformed.json")
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn("Failed to load json", proc.err)

    def test_submit_tuxbuild_missing(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/missing.json")
        self.assertFalse(proc.ok)
        self.assertIn(
            "No such file or directory: 'tests/data/submit/tuxbuild/missing.json'",
            proc.err,
        )

    def test_submit_tuxbuild_empty_build_status(self):
        proc = self.submit_tuxbuild(
            "tests/data/submit/tuxbuild/empty_build_status.json"
        )
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn(
            "Failed to validate tuxbuild data: '' is not one of ['fail', 'pass']",
            proc.err,
        )
        self.assertIn(
            "Failed validating 'enum' in schema['items'][0]['properties']['build_status']", proc.err
        )

    def test_submit_tuxbuild_malformed_build_status(self):
        proc = self.submit_tuxbuild(
            "tests/data/submit/tuxbuild/malformed_build_status.json"
        )
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn(
            "Failed to validate tuxbuild data: {'build': 'pass'} is not of type 'string'",
            proc.err,
        )
        self.assertIn(
            "Failed validating 'type' in schema['items'][0]['properties']['build_status']", proc.err
        )

    def test_submit_tuxbuild_missing_build_status(self):
        proc = self.submit_tuxbuild(
            "tests/data/submit/tuxbuild/missing_build_status.json"
        )
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn(
            "Failed to validate tuxbuild data: 'build_status' is a required property",
            proc.err,
        )

    def test_submit_tuxbuild_empty_kconfig(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/empty_kconfig.json")
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn("Failed to validate tuxbuild data: [] is too short", proc.err)
        self.assertIn(
            "Failed validating 'minItems' in schema['items'][0]['properties']['kconfig']", proc.err
        )

    def test_submit_tuxbuild_malformed_kconfig(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/malformed_kconfig.json")
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn(
            "Failed to validate tuxbuild data: {'CONFIG_ARM64_MODULE_PLTS': 'y'} is not of type 'string'",
            proc.err,
        )
        self.assertIn(
            "Failed validating 'type' in schema['items'][0]['properties']['kconfig']['items'][0]",
            proc.err,
        )

    def test_submit_tuxbuild_missing_kconfig(self):
        proc = self.submit_tuxbuild("tests/data/submit/tuxbuild/missing_kconfig.json")
        self.assertFalse(proc.ok, msg=proc.err)
        self.assertIn(
            "Failed to validate tuxbuild data: 'kconfig' is a required property",
            proc.err,
        )



from unittest import TestCase
from squad_client.utils import getid


class UtilsTest(TestCase):

    def test_getid(self):
        url = 'https://some-squad-url.com/api/objects/42/'
        self.assertEqual(42, getid(url))



#!/usr/bin/env python3

import argparse
import os
import sys


from squad_client import logging
from squad_client.core.api import SquadApi, ApiException
from squad_client.core.command import SquadClientCommand
from squad_client.commands import *  # noqa
from squad_client.version import __version__ as squad_client_version

logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(prog='./manage.py')
    parser.add_argument('--debug', action='store_true', help='display debug messages')
    parser.add_argument('--squad-host', help='SQUAD host, example: https://qa-reports.linaro.org')
    parser.add_argument('--squad-token', help='SQUAD authentication token')
    parser.add_argument('--cache', default=0, help='Cache API results for N number of seconds. Disabled by default.')
    parser.add_argument('--version', action='store_true', help='Display versions of squad-client and server')
    subparser = parser.add_subparsers(help='available subcommands', dest='command')

    SquadClientCommand.add_commands(subparser)

    args = parser.parse_args()

    if args.debug:
        logging.setLevel(logging.DEBUG)

    if args.command not in ['test']:
        squad_host = args.squad_host or os.getenv('SQUAD_HOST')
        squad_token = args.squad_token or os.getenv('SQUAD_TOKEN')
        if squad_host is None:
            logger.error('Either --squad-host or SQUAD_HOST env variable are required')
            return -1

        try:
            SquadApi.configure(squad_host, token=squad_token, cache=args.cache)
        except ApiException as e:
            logger.error('Failed to configure squad api: %s' % e)
            return -1

    if args.version:
        print('squad-client: %s' % squad_client_version)
        print('squad server: %s' % SquadApi.version)
        return 0

    if args.command is None:
        parser.print_help()
        return -1

    rc = SquadClientCommand.process(args)
    return 1 if rc is False else 0 if rc is True else -1


if __name__ == '__main__':
    sys.exit(main())



class SquadClientCommand:
    command = None
    help_text = None
    klasses = {}

    @staticmethod
    def add_commands(subparser):
        for klass in SquadClientCommand.__subclasses__():
            obj = klass()
            if obj.register(subparser) is None:
                SquadClientCommand.klasses[obj.command] = obj

    @staticmethod
    def process(args):
        return SquadClientCommand.klasses[args.command].run(args)

    def register(self, subparser):
        return subparser.add_parser(self.command, help=self.help_text)

    def run(args):
        raise NotImplementedError



import uuid
from itertools import groupby
from collections import OrderedDict


from .api import SquadApi, ApiException
from squad_client.exceptions import InvalidSquadObject, InvalidSquadLookup
from squad_client.utils import first, parse_test_name, parse_metric_name, to_json, get_class_name
from squad_client import settings
from squad_client import logging


logger = logging.getLogger(__name__)

DEFAULT_COUNT = settings.DEFAULT_NUM_OF_OBJECTS
ALL = -1


class SquadObjectException(Exception):
    pass


class SquadObject:
    endpoint = None
    attrs = []
    types = None

    def __init__(self, _id=None):
        if _id:
            self.endpoint += str(_id)
            self.__fetch__()

    @classmethod
    def get_type(cls, _type):
        if SquadObject.types is None:
            SquadObject.types = {c.__name__: c for c in SquadObject.__subclasses__() if c is not Squad}

        returned_type = SquadObject.types.get(_type)
        if returned_type is None:
            raise InvalidSquadObject('There is no SquadObject of type "%s"' % _type)

        return returned_type

    @property
    def __id__(self):
        if not hasattr(self, 'id'):
            setattr(self, 'id', None)

        if self.id is None:
            self.id = uuid.uuid1()

        return self.id

    def __fill_object__(self, result, obj=None):

        if obj is None:
            obj = self

        attrs = obj.attrs if len(obj.attrs) else [attr for attr in result.keys()]
        for attr in attrs:
            if attr in result.keys():
                setattr(obj, attr.replace(' ', '_').replace('/', '_').replace('-', '_'), result[attr])

    def __fill__(self, klass, results):

        objects = {}
        for result in results:
            obj = klass()
            self.__fill_object__(result, obj)
            objects[obj.__id__] = obj

        return objects

    def __str__(self):
        class_name = get_class_name(self)
        attrs_str = []
        for attr in self.attrs:
            attrs_str.append('%s: "%s"' % (attr, getattr(self, attr) if hasattr(self, attr) else None))

        return '%s(%s)' % (class_name, ', '.join(attrs_str))

    def __fetch__(self, klass=None, filters=None, count=DEFAULT_COUNT, endpoint=None):
        """
            Generic get method to retrieve objects from API
            count: number of objects to fetch, defaults to 50,
                      -1 (ALL) means follow pagination
        """

        if klass is None:
            response = SquadApi.get(self.endpoint)
            result = response.json()
            self.__fill_object__(result)
            return

        if count == ALL:
            count = settings.MAX_NUM_OF_OBJECTS

        filters['limit'] = count if count < settings.SQUAD_MAX_PAGE_LIMIT else settings.SQUAD_MAX_PAGE_LIMIT
        objects = {}
        url = endpoint or klass.endpoint
        while url and len(objects) < count:
            response = SquadApi.get(url, filters)
            result = response.json()
            url = result['next']
            objects.update(self.__fill__(klass, result['results']))

        if len(objects) > settings.MAX_NUM_OF_OBJECTS:
            logger.warn('Maximum number of objects reached [%d]!' % len(objects))

        return objects

    def get(self, _id):
        count = 1
        result = self.__fetch__(self.__class__, {'id': _id}, count)
        return first(result) if len(result) else None

    def pre_save(self):
        pass

    def post_save(self):
        pass

    def save(self):
        self.pre_save()

        data = {}
        class_name = get_class_name(self)

        for attr in self.attrs:
            if not hasattr(self, attr):
                continue

            value = getattr(self, attr, None)
            if value is None:
                continue

            if isinstance(value, SquadObject):
                # value is a relation object
                if not hasattr(value, 'id'):
                    raise SquadObjectException('Failed to save %s: %s has no id' % (class_name, attr))
                # TODO: some objects have url as reference to other objects, ex:
                # project.group is a group url, instead of id
                # we need to standardize it
                # For now, only projects can be created this way
                data[attr] = value.url
            else:
                data[attr] = value

        endpoint = self.endpoint
        request = SquadApi.post
        if hasattr(self, 'id') and type(self.id) is not uuid.UUID:
            endpoint = '%s%s/' % (endpoint, self.id)
            request = SquadApi.patch

        try:
            response = request(endpoint, data=data)
            if response.status_code in [400, 401, 405]:
                raise SquadObjectException('Failed to save %s: %s' % (class_name, response.text))

            self.__fill_object__(response.json())
            self.post_save()

        except ApiException as e:
            logger.error(e)
            raise SquadObjectException(str(e))

    def delete(self):
        class_name = get_class_name(self)

        if not hasattr(self, 'id') or type(self.id) is uuid.UUID:
            raise SquadObjectException('Failed to delete %s: it must contain a valid "id"' % class_name)

        endpoint = '%s%s/' % (self.endpoint, self.id)

        try:
            response = SquadApi.delete(endpoint)
            if response.status_code in [400, 401, 405]:
                raise SquadObjectException('Failed to delete %s: %s' % (class_name, response.text))

        except ApiException as e:
            logger.error(e)
            raise SquadObjectException(str(e))


class Squad(SquadObject):

    def fetch(self, klass, count=ALL, **filters):
        return self.__fetch__(klass, filters, count)

    def groups(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Group, filters, count)

    def group(self, slug, **filters):
        filters.update({'slug': slug})
        objects = self.groups(count=1, **filters)
        return first(objects)

    def projects(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Project, filters, count)

    def builds(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Build, filters, count)

    def testjobs(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(TestJob, filters, count)

    def testruns(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(TestRun, filters, count)

    def tests(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Test, filters, count)

    def metrics(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Metric, filters, count)

    def suites(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Suite, filters, count)

    def environments(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Environment, filters, count)

    def backends(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Backend, filters, count)

    def emailtemplates(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(EmailTemplate, filters, count)

    def knownissues(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(KnownIssue, filters, count)

    def suitemetadata(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(SuiteMetadata, filters, count)

    def annotations(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Annotation, filters, count)

    def metricthresholds(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(MetricThreshold, filters, count)

    def reports(self, count=DEFAULT_COUNT, **filters):
        return self.__fetch__(Report, filters, count)

    def submit(self, group=None, project=None, build=None, environment=None,
               tests=None, metrics=None, metadata=None, log=None, attachments=None):

        path = '/api/submit/%s/%s/%s/%s' % (group.slug, project.slug, build.version, environment.slug)
        num_tests = 0
        num_metrics = 0

        data = {}
        if tests:
            tests_dict = {}
            for test in tests.values():
                if hasattr(test, 'log') and test.log is not None and len(test.log):
                    value = {'log': test.log, 'result': test.status}
                else:
                    value = test.status
                tests_dict[test.name] = value
            num_tests = len(tests_dict)
            data['tests'] = to_json(tests_dict)
        if metrics:
            metrics_dict = {metric.name: metric.result for metric in metrics.values()}
            num_metrics = len(metrics_dict)
            data['metrics'] = to_json(metrics_dict)
        if metadata:
            data['metadata'] = to_json(metadata)
        if log:
            data['log'] = log

        logger.info('Submitting %i tests, %i metrics' % (num_tests, num_metrics))

        # TODO handle attachments
        response = SquadApi.post(path, data=data)
        status_code = response.status_code
        if status_code not in [200, 201, 500]:
            logger.error('Failed to submit results: %s' % response.text)
        return response.ok


class Group(SquadObject):

    endpoint = '/api/groups/'
    attrs = ['id', 'url', 'slug', 'name', 'description']

    def projects(self, count=DEFAULT_COUNT, **filters):
        filters.update({'group': self.id})
        return self.__fetch__(Project, filters, count)

    def project(self, slug):
        filters = {'slug': slug}
        objects = self.projects(count=1, **filters)
        return first(objects)

    def create_project(self, slug=None, plugins_list=None):
        new_project = Project()
        new_project.slug = slug
        new_project.group = self
        new_project.enabled_plugins_list = plugins_list or ['linux-log-parser']
        new_project.save()

    def __repr__(self):
        return self.slug


class Project(SquadObject):

    endpoint = '/api/projects/'
    attrs = ['id', 'custom_email_template', 'data_retention_days', 'description',
             'enabled_plugins_list', 'full_name', 'group', 'html_mail', 'important_metadata_keys',
             'is_archived', 'is_public', 'moderate_notifications', 'name', 'notification_timeout',
             'slug', 'url', 'wait_before_notification']

    def builds(self, count=DEFAULT_COUNT, **filters):
        filters.update({'project': self.id})
        return self.__fetch__(Build, filters, count)

    def build(self, version):
        filters = {'version': version}
        objects = self.builds(count=1, **filters)
        return first(objects)

    def environments(self, count=DEFAULT_COUNT, **filters):
        filters.update({'project': self.id})
        return self.__fetch__(Environment, filters, count)

    def environment(self, slug):
        filters = {'slug': slug}
        objects = self.environments(count=1, **filters)
        return first(objects)

    def suites(self, count=DEFAULT_COUNT, **filters):
        filters.update({'project': self.id})
        return self.__fetch__(Suite, filters, count)

    def suite(self, suite_slug):
        filters = {'slug': suite_slug}
        objects = self.suites(count=1, **filters)
        return first(objects)

    def __repr__(self):
        return self.slug

    @staticmethod
    def compare_builds(baseline_id, build_id, force=False):
        try:
            int(baseline_id)
            int(build_id)
        except ValueError:
            raise ValueError("IDs must be valid integers")
        baseline = Build(baseline_id)
        to_compare = Build(build_id)
        if baseline.id and to_compare.id:
            proj_id = baseline.project.split("/")[-2]
            if proj_id != to_compare.project.split("/")[-2]:
                raise InvalidSquadLookup("Argument builds must belong to same project")
            url = ''.join([Project.endpoint, str(proj_id), '/compare_builds'])
            params = {'baseline': baseline_id, 'to_compare': build_id}
            if force:
                params['force'] = '1'
            return SquadApi.get(url, params).json()

    def pre_save(self):
        self.attrs.append('project_settings')

        if not hasattr(self, 'enabled_plugins_list'):
            # TODO: make enabled_plugins_list optional
            self.enabled_plugins_list = ['linux-log-parser']


class Build(SquadObject):

    endpoint = '/api/builds/'
    attrs = ['url', 'id', 'testjobs', 'finished',
             'version', 'created_at', 'datetime', 'patch_id', 'keep_data', 'project',
             'patch_source', 'patch_baseline']

    def testruns(self, count=ALL, bucket_suites=False, **filters):
        filters.update({'build': self.id})
        testruns = self.__fetch__(TestRun, filters, count)

        if bucket_suites:
            for _id in testruns.keys():
                testruns[_id].bucket_metric_and_test_suites()

        return testruns

    __tests__ = {}

    def tests(self, count=ALL, **filters):
        filters['count'] = count
        filters_str = str(OrderedDict(filters))
        if self.__tests__.get(filters_str) is None:
            endpoint = '%s%d/tests/' % (self.endpoint, self.id)
            self.__tests__[filters_str] = self.__fetch__(Test, filters, count, endpoint=endpoint)
        return self.__tests__[filters_str]

    __metadata__ = None
    __status__ = None

    @property
    def metadata(self):
        if self.__metadata__ is None:
            endpoint = '%s%d/metadata' % (self.endpoint, self.id)
            response = SquadApi.get(endpoint)
            objects = self.__fill__(BuildMetadata, [response.json()])
            self.__metadata__ = first(objects)
        return self.__metadata__

    @property
    def status(self):
        if self.__status__ is None:
            endpoint = '%s%d/status' % (self.endpoint, self.id)
            response = SquadApi.get(endpoint)
            objects = self.__fill__(BuildStatus, [response.json()])
            self.__status__ = first(objects)
        return self.__status__

    def __repr__(self):
        return self.version


class BuildMetadata(SquadObject):
    pass


class BuildStatus(SquadObject):
    pass


class TestJob(SquadObject):

    endpoint = '/api/testjobs/'
    attrs = ['url', 'external_url', 'definition', 'name', 'environment', 'created_at',
             'submitted_at', 'fetched_at', 'submitted', 'fetched', 'fetch_attempts',
             'last_fetch_attempt', 'failure', 'can_resubmit', 'resubmitted_count',
             'job_id', 'job_status', 'backend', 'testrun', 'target', 'target_build',
             'parent_job']


class MetricSuite:
    name = ''
    __metrics__ = {}

    def add_metric(self, metric):
        if self.__metric__ is None:
            self.__metric__ = {}
        self.__metrics__[metric.id] = metric

    @property
    def metrics(self):
        return self.__metrics__


class Metric(SquadObject):
    endpoint = '/api/metrics/'
    attrs = ['url', 'id', 'name', 'short_name', 'measurement_list', 'result', 'unit', 'is_outlier', 'test_run', 'suite', 'metadata']


class TestRunStatus(SquadObject):
    attrs = ['url', 'id', 'tests_pass', 'tests_fail', 'tests_xfail',
             'tests_skip', 'metrics_summary', 'has_metrics',
             'suite', 'suite_version']


class TestRun(SquadObject):

    endpoint = '/api/testruns/'
    attrs = ['url', 'id', 'metadata_file', 'log_file',
             'created_at', 'completed', 'datetime', 'build_url',
             'job_id', 'job_status', 'job_url', 'resubmit_url',
             'data_processed', 'status_recorded', 'build',
             'environment']
    metadata = None
    attachments = None
    log = None

    __tests__ = None

    def add_test(self, test):
        if self.__tests__ is None:
            self.__tests__ = {}
        self.__tests__[test.__id__] = test

    def tests(self, count=ALL, **filters):
        if self.__tests__ is None and hasattr(self, 'id') and self.id is not None:
            filters.update({'test_run': self.id})
            self.__tests__ = self.__fetch__(Test, filters, count)
        return self.__tests__

    __metrics__ = None

    def add_metric(self, metric):
        if self.__metrics__ is None:
            self.__metrics__ = {}
        self.__metrics__[metric.__id__] = metric

    def metrics(self, count=ALL, **filters):
        if self.__metrics__ is None and hasattr(self, 'id') and self.id is not None:
            filters.update({'test_run': self.id})
            self.__metrics__ = self.__fetch__(Metric, filters, count)
        return self.__metrics__

    test_suites = []
    metric_suites = []

    def bucket_metric_and_test_suites(self):
        all_tests = self.tests()
        self.test_suites = []
        if len(all_tests):
            for suite_name, tests in groupby(sorted(all_tests.values(), key=lambda t: t.name), lambda t: parse_test_name(t.name)[0]):
                test_suite = Suite()
                test_suite.name = suite_name
                self.test_suites.append(test_suite)
                for test in tests:
                    test_suite.add_test(test)

        all_metrics = self.metrics()
        if len(all_metrics):
            self.metric_suites = []
            for suite_name, metrics in groupby(sorted(all_metrics.values(), key=lambda m: m.name), lambda m: parse_metric_name(m.name)[0]):
                metric_suite = MetricSuite()
                metric_suite.name = suite_name
                [metric_suite.add_metric(m) for m in metrics]
                self.metric_suites.append(metric_suite)

    def submit_results(self):
        squad = Squad()
        return squad.submit(
            group=self.build.project.group,
            project=self.build.project,
            build=self.build,
            environment=self.environment,
            tests=self.tests(),
            metrics=self.metrics(),
            metadata=self.metadata,
            log=self.log,
            attachments=self.attachments)

    __summary__ = None

    def summary(self):
        if self.__summary__ is None:
            self.__summary__ = first(self.statuses(suite__isnull=True))
        return self.__summary__

    def statuses(self, count=ALL, **filters):
        TestRunStatus.endpoint = '/'.join([TestRun.endpoint[:-1], str(self.id), 'status'])
        return self.__fetch__(TestRunStatus, filters, count)


class Test(SquadObject):

    endpoint = '/api/tests/'
    attrs = ['url', 'id', 'name', 'short_name', 'status', 'result', 'test_run', 'log', 'has_known_issues',
             'suite', 'known_issues', 'build', 'environment']

    def __repr__(self):
        return self.short_name


class Suite(SquadObject):

    endpoint = '/api/suites/'
    attrs = ['url', 'id', 'slug', 'name', 'project']

    __tests__ = None

    def add_test(self, test):
        if self.__tests__ is None:
            self.__tests__ = {}
        self.__tests__[test.__id__] = test

    def tests(self, count=ALL, **filters):
        if self.__tests__ is None and hasattr(self, 'id') and self.id is not None:
            endpoint = '%s%d/tests' % (self.endpoint, self.id)
            self.__tests__ = self.__fetch__(Test, filters, count, endpoint=endpoint)
        return self.__tests__

    def __repr__(self):
        return self.slug


class Environment(SquadObject):

    endpoint = '/api/environments/'
    attrs = ['url', 'id', 'slug', 'name', 'expected_test_runs', 'description', 'project']

    def __repr__(self):
        return self.slug


class Backend(SquadObject):

    endpoint = '/api/backends/'
    attrs = ['id', 'name', 'url', 'username', 'implementation_type', 'backend_settings',
             'poll_interval', 'max_fetch_attempts', 'poll_enabled', 'listen_enabled']


class EmailTemplate(SquadObject):

    endpoint = '/api/emailtemplates/'
    attrs = ['url', 'id', 'name', 'subject', 'plain_text', 'html']


class KnownIssue(SquadObject):

    endpoint = '/api/knownissues/'
    attrs = ['url', 'id', 'title', 'test_name', 'notes', 'active', 'intermittent', 'environments']


class SuiteMetadata(SquadObject):

    endpoint = '/api/suitemetadata/'
    attrs = ['url', 'id', 'name', 'suite', 'kind', 'description', 'instructions_to_reproduce']


class Annotation(SquadObject):

    endpoint = '/api/annotations/'
    attrs = ['url', 'id', 'description', 'build']


class MetricThreshold(SquadObject):

    endpoint = '/api/metricthresholds/'
    attrs = ['url', 'id', 'name', 'value', 'is_higher_better', 'environment']


class Report(SquadObject):

    endpoint = '/api/reports/'
    attrs = ['url', 'id', 'baseline', 'output_format', 'email_recipient', 'email_recipient_notified',
             'callback', 'callback_notified', 'data_retention_days', 'output_subject', 'output_text',
             'output_html', 'error_message', 'status_code', 'created_at', 'build', 'template']



from unittest import TestCase

from . import settings
from squad_client.core.api import SquadApi, ApiException


class SquadApiTest(TestCase):

    def setUp(self):
        SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT)

    def test_unconfigured(self):
        with self.assertRaises(ApiException):
            SquadApi.configure(None)

    def test_malformed_url(self):
        with self.assertRaises(ApiException):
            SquadApi.configure('http:/malformed/url')

    def test_out_of_domain_object_url(self):
        with self.assertRaises(ApiException):
            SquadApi.get('http://some.other.url')

    def test_unauthorized_access(self):
        with self.assertRaises(ApiException):
            SquadApi.get('/my_group/my_private_project')

        SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT, token='193cd8bb41ab9217714515954e8724f651ef8601')
        response = SquadApi.get('/my_group/my_private_project')
        self.assertTrue(response.ok)

        # reset config
        self.setUp()



import hashlib
import json
import jsonschema
import os

from squad_client import logging
from squad_client.shortcuts import submit_results
from squad_client.core.command import SquadClientCommand


logger = logging.getLogger(__name__)


tuxbuild_schema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "array",
    "minItems": 1,
    "items": [{
        "type": "object",
        "properties": {
            "build_status": {
                "type": "string",
                "enum": ["fail", "pass"],
            },
            "git_describe": {
                "type": "string",
            },
            "kconfig": {
                "type": "array",
                "minItems": 1,
                "uniqueItems": True,
                "items": [{"type": "string"}],
            },
            "target_arch": {
                "type": "string",
            },
            "toolchain": {
                "type": "string",
            },
        },
        "required": [
            "build_status",
            "git_describe",
            "kconfig",
            "target_arch",
            "toolchain",
        ],
    }],
}

ALLOWED_METADATA = [
    "git_branch", "git_commit", "git_describe", "git_ref", "git_repo", "git_sha", "git_short_log", "kernel_version", "make_kernelversion",
]


class SubmitTuxbuildCommand(SquadClientCommand):
    command = "submit-tuxbuild"
    help_text = "submit tuxbuild results to SQUAD"

    def register(self, subparser):
        parser = super(SubmitTuxbuildCommand, self).register(subparser)
        parser.add_argument(
            "--group", help="SQUAD group where results are stored", required=True
        )
        parser.add_argument(
            "--project", help="SQUAD project where results are stored", required=True
        )
        parser.add_argument(
            "tuxbuild",
            help="File with tuxbuild results to submit",
        )

    def _build_metadata(self, builds):
        metadata = {k: v for k, v in builds[0].items() if k in ALLOWED_METADATA}

        # We expect git_commit, but tuxmake calls it git_sha
        metadata.update({"git_commit": metadata.get("git_sha")})

        # We expect `git_branch`, but tuxmake calls it `git_ref`
        # `git_ref` will sometimes be null
        metadata.update({"git_branch": metadata.get("git_ref")})

        # If `git_ref` is null, use `KERNEL_BRANCH` from the CI environment
        if metadata.get("git_branch") is None:
            metadata.update({"git_branch": os.getenv("KERNEL_BRANCH")})

        # We expect `make_kernelversion`, but tuxmake calls it `kernel_version`
        metadata.update({"make_kernelversion": metadata.get("kernel_version")})

        return metadata

    def _load_builds(self, path):
        builds = None
        try:
            with open(path) as f:
                builds = json.load(f)

        except json.JSONDecodeError as jde:
            logger.error("Failed to load json: %s", jde)

        except OSError as ose:
            logger.error("Failed to open file: %s", ose)

        return builds

    def _get_test_name(self, kconfig, toolchain):
        if len(kconfig[1:]):
            kconfig_hash = "%s-%s" % (
                kconfig[0],
                hashlib.sha1(json.dumps(kconfig[1:]).encode()).hexdigest()[0:8],
            )
        else:
            kconfig_hash = kconfig[0]

        return "build/%s-%s" % (toolchain, kconfig_hash)

    def run(self, args):
        builds = self._load_builds(args.tuxbuild)

        # log
        if builds is None:
            return False

        try:
            jsonschema.validate(instance=builds, schema=tuxbuild_schema)
        except jsonschema.exceptions.ValidationError as ve:
            logger.error("Failed to validate tuxbuild data: %s", ve)
            return False

        data = {}
        for build in builds:
            arch = build["target_arch"]
            description = build["git_describe"]
            kconfig = build["kconfig"]
            status = build["build_status"]
            toolchain = build["toolchain"]
            test = self._get_test_name(kconfig, toolchain)

            multi_key = (description, arch)
            if multi_key not in data:
                data[multi_key] = {}

            data[multi_key].update({test: status})

        for key, result in data.items():
            description, arch = key
            submit_results(
                group_project_slug="%s/%s" % (args.group, args.project),
                build_version=description,
                env_slug=arch,
                tests=result,
                metadata=self._build_metadata(builds),
            )

        return True



from unittest import TestCase
import subprocess as sp


from tests import settings
from squad_client.core.api import SquadApi
from squad_client.core.models import Squad
from squad_client.utils import first


class CreateOrUpdateProjectTest(TestCase):
    def setUp(self):
        self.squad = Squad()

        self.testing_server = 'http://localhost:%s' % settings.DEFAULT_SQUAD_PORT
        self.testing_token = '193cd8bb41ab9217714515954e8724f651ef8601'
        SquadApi.configure(self.testing_server, self.testing_token)

        self.group = 'my_group'
        self.slug = 'create-project-via-cmdline'

    def manage_create_or_update_project(self, group=None, slug=None, name=None, description=None, settings=None, is_public=None, html_mail=None,
                                        moderate_notifications=None, is_archived=None, email_template=None,
                                        plugins=None, important_metadata_keys=None, wait_before_notification_timeout=None,
                                        notification_timeout=None, data_retention=None, no_overwrite=False):
        argv = ['./manage.py', '--squad-host', self.testing_server, '--squad-token', self.testing_token,
                'create-or-update-project']

        if group:
            argv += ['--group', group]
        if slug:
            argv += ['--slug', slug]
        if name:
            argv += ['--name', name]
        if description:
            argv += ['--description', description]
        if settings:
            argv += ['--settings', settings]
        if is_public is not None:
            argv += ['--is-public'] if is_public else ['--is-private']
        if html_mail is not None:
            argv += ['--html-mail'] if html_mail else ['--no-html-mail']
        if moderate_notifications is not None:
            argv += ['--moderate-notifications'] if moderate_notifications else ['--no-moderate-notifications']
        if is_archived is not None and is_archived:
            argv += ['--is-archived']
        if email_template:
            argv += ['--email-template', email_template]
        if plugins and len(plugins):
            argv += ['--plugins', ','.join(plugins)]
        if important_metadata_keys and len(important_metadata_keys):
            argv += ['--important-metadata-keys', ','.join(important_metadata_keys)]
        if wait_before_notification_timeout is not None:
            argv += ['--wait-before-notification-timeout', str(wait_before_notification_timeout)]
        if notification_timeout is not None:
            argv += ['--notification-timeout', str(notification_timeout)]
        if data_retention is not None:
            argv += ['--data-retention', str(data_retention)]
        if no_overwrite:
            argv += ['--no-overwrite']

        proc = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE)
        proc.ok = False

        try:
            out, err = proc.communicate()
            proc.ok = (proc.returncode == 0)
        except sp.TimeoutExpired:
            self.logger.error('Running "%s" time out after %i seconds!' % ' '.join(argv))
            proc.kill()
            out, err = proc.communicate()

        proc.out = out.decode('utf-8')
        proc.err = err.decode('utf-8')
        return proc

    def test_empty(self):
        proc = self.manage_create_or_update_project()
        self.assertFalse(proc.ok)
        self.assertIn('the following arguments are required: --group, --slug', proc.err)

    def test_basics(self):
        proc = self.manage_create_or_update_project(group=self.group, slug=self.slug)
        self.assertTrue(proc.ok)

        project = first(self.squad.projects(group__slug=self.group, slug=self.slug))
        self.assertIsNotNone(project)
        self.assertIn('Project saved', proc.out)

    def test_no_overwrite(self):
        proc = self.manage_create_or_update_project(group=self.group, slug=self.slug)
        self.assertTrue(proc.ok)

        proc = self.manage_create_or_update_project(group=self.group, slug=self.slug, name='trying to edit', no_overwrite=True)
        self.assertFalse(proc.ok)
        self.assertIn('Project exists already', proc.err)

    def test_all_parameters(self):
        name = 'new name'
        description = 'project description'
        settings = '{"SETTING_KEY": "SETTING VALUE"}'
        is_public = True
        html_mail = False
        moderate_notifications = False
        is_archived = False
        plugins = ['linux-log-parser']
        important_metadata_keys = ['important-key-1', 'important key 2']
        wait_before_notification_timeout = 60
        notification_timeout = 120
        data_retention = 1

        proc = self.manage_create_or_update_project(
            group=self.group,
            slug=self.slug,
            name=name,
            description=description,
            settings=settings,
            is_public=is_public,
            html_mail=html_mail,
            moderate_notifications=moderate_notifications,
            is_archived=is_archived,
            plugins=plugins,
            important_metadata_keys=important_metadata_keys,
            wait_before_notification_timeout=wait_before_notification_timeout,
            notification_timeout=notification_timeout,
            data_retention=data_retention,
        )
        self.assertTrue(proc.ok)

        project = first(self.squad.projects(group__slug=self.group, slug=self.slug))
        self.assertIsNotNone(project)
        self.assertIn('Project saved', proc.out)

        self.assertEqual(description, project.description)
        self.assertEqual(settings, project.project_settings)
        self.assertEqual(is_public, project.is_public)
        self.assertEqual(html_mail, project.html_mail)
        self.assertEqual(moderate_notifications, project.moderate_notifications)
        self.assertEqual(is_archived, project.is_archived)
        self.assertEqual(plugins, project.enabled_plugins_list)
        self.assertEqual('\n'.join(important_metadata_keys), project.important_metadata_keys)
        self.assertEqual(wait_before_notification_timeout, project.wait_before_notification)
        self.assertEqual(notification_timeout, project.notification_timeout)
        self.assertEqual(data_retention, project.data_retention_days)



import logging
import os


DEBUG = logging.DEBUG
ERROR = logging.ERROR
INFO = logging.INFO
levels_str = {'DEBUG': DEBUG, 'ERROR': ERROR, 'INFO': INFO, None: None}


level_env = os.getenv('LOG_LEVEL')
if level_env not in levels_str:
    level_env = None
level_env = levels_str[level_env]


_level = level_env or INFO
_loggers = []


def getLogger(name=None):
    logger = logging.getLogger(name) if name else logging.getLogger()
    logger.setLevel(_level)

    formatter = logging.Formatter('[%(levelname)s] %(name)s: %(message)s')
    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    _loggers.append(logger)

    return logger


def setLevel(level):
    _level = level
    for logger in _loggers:
        logger.setLevel(_level)



import unittest
import subprocess as sp
import os

from . import settings
from squad_client.core.api import SquadApi
from squad_client.core.models import Squad, Group, Project, Build, Environment, Test, TestRun, Metric
from squad_client.utils import first


PASS = 'pass'
FAIL = 'fail'
XFAIL = 'xfail'
SKIP = 'skip'


class SquadSubmitTest(unittest.TestCase):

    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT, token='193cd8bb41ab9217714515954e8724f651ef8601')

    def test_submit(self):
        group = Group()
        group.slug = 'my_group'

        project = Project()
        project.slug = 'my_project'
        project.group = group

        env = Environment()
        env.slug = 'my_env'
        env.project = project

        build = Build()
        build.project = project
        build.version = 'my_build'

        testrun = TestRun()
        testrun.build = build

        test = Test()
        test.name = 'test1'
        test.status = PASS
        test.log = 'test1 log'

        metric = Metric()
        metric.name = 'metric1'
        metric.result = 42

        testrun.environment = env
        testrun.add_test(test)
        testrun.add_metric(metric)
        testrun.log = 'really long log'
        testrun.metadata = {'metadata1': 'value1', 'metadata2': 'value2', 'job_id': '123'}

        testrun.submit_results()

        results = self.squad.tests(name='test1')
        self.assertTrue(len(results) > 0)
        t = first(results)
        self.assertEqual(t.log, test.log)
        self.assertEqual(t.name, test.name)


class SubmitCommandTest(unittest.TestCase):

    testing_server = 'http://localhost:%s' % settings.DEFAULT_SQUAD_PORT
    testing_token = '193cd8bb41ab9217714515954e8724f651ef8601'

    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(url=self.testing_server, token=self.testing_token)

    def manage_submit(self, results=None, result_name=None, result_value=None, metrics=None,
                      metadata=None, attachments=None, logs=None, environment=None):
        argv = ['./manage.py', '--squad-host', self.testing_server, '--squad-token', self.testing_token,
                'submit', '--group', 'my_group', '--project', 'my_project', '--build', 'my_build6', '--environment', 'test_submit_env']

        if logs:
            argv += ['--logs', logs]
        if results:
            argv += ['--results', results]
        if metrics:
            argv += ['--metrics', metrics]
        if metadata:
            argv += ['--metadata', metadata]
        if attachments:
            argv += ['--attachments', attachments]
        if result_name:
            argv += ['--result-name', result_name]
        if result_value:
            argv += ['--result-value', result_value]

        env = os.environ.copy()
        env['LOG_LEVEL'] = 'INFO'
        proc = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE, env=env)
        proc.ok = False

        try:
            out, err = proc.communicate()
            proc.ok = (proc.returncode == 0)
        except sp.TimeoutExpired:
            self.logger.error('Running "%s" time out after %i seconds!' % ' '.join(argv))
            proc.kill()
            out, err = proc.communicate()

        proc.out = out.decode('utf-8')
        proc.err = err.decode('utf-8')
        return proc

    def test_submit_empty(self):
        proc = self.manage_submit()
        self.assertFalse(proc.ok)
        self.assertIn('At least one of --result-name, --results, --metrics is required', proc.err)

    def test_submit_single_test(self):
        proc = self.manage_submit(result_name='single-test', result_value='pass')
        self.assertTrue(proc.ok)
        self.assertIn('1 tests', proc.err)

        test = first(self.squad.tests(name='single-test'))
        self.assertEqual('single-test', test.name)
        self.assertEqual('pass', test.status)

    def test_submit_invalid_result_value(self):
        proc = self.manage_submit(result_name='single-invalid-test', result_value='not-valid')
        self.assertFalse(proc.ok)
        self.assertIn("result-value: invalid choice: 'not-valid'", proc.err)

    def test_submit_results_json(self):
        proc = self.manage_submit(results='tests/submit_results/sample_results.json')
        self.assertTrue(proc.ok)
        self.assertIn('2 tests', proc.err)

        test = first(self.squad.tests(name='json-test-1'))
        self.assertEqual('json-test-1', test.name)
        self.assertEqual('pass', test.status)

        test = first(self.squad.tests(name='json-test-2'))
        self.assertEqual('json-test-2', test.name)
        self.assertEqual('fail', test.status)
        self.assertEqual('json-test-2 log', test.log)

    def test_submit_results_malformed_json(self):
        proc = self.manage_submit(results='tests/submit_results/sample_results_malformed.json')
        self.assertFalse(proc.ok)
        self.assertIn('Failed parsing file', proc.err)

    def test_submit_results_yaml(self):
        proc = self.manage_submit(results='tests/submit_results/sample_results.yaml')
        self.assertTrue(proc.ok)
        self.assertIn('2 tests', proc.err)

        test = first(self.squad.tests(name='yaml-test-1'))
        self.assertEqual('yaml-test-1', test.name)
        self.assertEqual('pass', test.status)

        test = first(self.squad.tests(name='yaml-test-2'))
        self.assertEqual('yaml-test-2', test.name)
        self.assertEqual('fail', test.status)
        self.assertEqual('yaml-test-2 log', test.log)

    def test_submit_results_malformed_yaml(self):
        proc = self.manage_submit(results='tests/submit_results/sample_results_malformed.yaml')
        self.assertFalse(proc.ok)
        self.assertIn('Failed parsing file', proc.err)

    def test_submit_results_bad_extension(self):
        p = "tests/submit_results/sample_results.txt"
        proc = self.manage_submit(results=p)
        self.assertFalse(proc.ok)
        self.assertIn('File "%s" does not have a JSON or YAML file extension' % p, proc.err)

    def test_submit_single_metric(self):
        proc = self.manage_submit(metrics='tests/submit_results/sample_metrics.json')
        self.assertTrue(proc.ok)
        self.assertIn('1 metrics', proc.err)

    def test_submit_everything(self):
        proc = self.manage_submit(results='tests/submit_results/sample_results.json',
                                  metrics='tests/submit_results/sample_metrics.json',
                                  metadata='tests/submit_results/sample_metadata.json',
                                  logs='tests/submit_results/sample_log.log')
        self.assertTrue(proc.ok, msg=proc.err)
        self.assertIn('2 tests, 1 metrics', proc.err)

        testrun = first(self.squad.testruns(job_id='jsonmetadatajobid1'))
        self.assertEqual('jsonmetadatajobid1', testrun.job_id)

        self.assertEqual(2, len(testrun.tests()))
        self.assertEqual(1, len(testrun.metrics()))



# Maximum number of objects to fetch from a unique endpoint
# this will avoid infinite loop
MAX_NUM_OF_OBJECTS = 10e6

# Default number of objects to load for a endpoint
DEFAULT_NUM_OF_OBJECTS = 50

# Maximum number of objects loaded per page in SQUAD
SQUAD_MAX_PAGE_LIMIT = 1000



import sys

from squad_client import logging
from squad_client.shortcuts import create_or_update_project
from squad_client.core.command import SquadClientCommand


logger = logging.getLogger(__name__)


class CreateOrUpdateProjectCommand(SquadClientCommand):
    command = "create-or-update-project"
    help_text = "Create or update a project in SQUAD"

    def register(self, subparser):
        parser = super(CreateOrUpdateProjectCommand, self).register(subparser)
        parser.add_argument(
            "--group",
            help="SQUAD group where results are stored",
            required=True,
        )
        parser.add_argument(
            "--slug",
            help="SQUAD project slug",
            required=True,
        )
        parser.add_argument(
            "--name",
            help="Project name",
        )
        parser.add_argument(
            "--description",
            help="Project description",
        )
        parser.add_argument(
            '--settings',
            help="Project settings in yaml/json",
        )

        privacy = parser.add_mutually_exclusive_group()
        privacy.add_argument('--is-public', action='store_true', help='Project is public')
        privacy.add_argument('--is-private', action='store_true', help='Project is private')

        html_emails = parser.add_mutually_exclusive_group()
        html_emails.add_argument('--html-mail', action='store_true', help='Enable html emails')
        html_emails.add_argument('--no-html-mail', action='store_true', help='Disable html emails')

        moderate_notifications = parser.add_mutually_exclusive_group()
        moderate_notifications.add_argument('--moderate-notifications', action='store_true', help='Enable moderating notifications')
        moderate_notifications.add_argument('--no-moderate-notifications', action='store_true', help='Disable moderating notifications')

        parser.add_argument(
            "--email-template",
            help="Email template name to use, use SQUAD's default if none is given",
        )
        parser.add_argument(
            "--plugins",
            help="Plugins to enable separated by comma: [ ltp | linux-log-parser | tradefed ]",
            # TODO: auto-generate list of available plugins using squadplugins
        )
        parser.add_argument(
            "--important-metadata-keys",
            help="Important metadata keys separated by comma",
        )
        parser.add_argument(
            "--wait-before-notification-timeout",
            help="Wait this many seconds before sending notifications",
            type=int,
        )
        parser.add_argument(
            "--notification-timeout",
            help="Force sending build notifications after this many seconds",
            type=int,
        )
        parser.add_argument(
            "--data-retention",
            help="Delete builds older than this number of days. Set to 0 or any negative number to disable",
            type=int,
            default=0,
        )
        parser.add_argument(
            '--is-archived',
            action='store_true',
            default=False,
            help="Makes the project hidden from the group page by default",
        )
        parser.add_argument(
            '--no-overwrite',
            action='store_true',
            default=False,
            help="Command will fail if trying to update a project that already exists",
        )
        parser.add_argument(
            '--silent',
            action='store_true',
            default=False,
            help="Return with exit code only, do not print errors whatsoever",
        )

    def resolve_boolean_flag(self, flag, no_flag):
        if flag:
            return True
        if no_flag:
            return False
        return None

    def run(self, args):
        is_public = self.resolve_boolean_flag(args.is_public, args.is_private)
        html_mail = self.resolve_boolean_flag(args.html_mail, args.no_html_mail)
        moderate_notifications = self.resolve_boolean_flag(args.moderate_notifications, args.no_moderate_notifications)
        plugins = args.plugins.split(',') if args.plugins else None
        important_metadata_keys = args.important_metadata_keys.split(',') if args.important_metadata_keys else None

        project, errors = create_or_update_project(
            group_slug=args.group,
            slug=args.slug,
            name=args.name,
            settings=args.settings,
            is_public=is_public,
            html_mail=html_mail,
            is_archived=args.is_archived,
            description=args.description,
            plugins=plugins,
            data_retention=args.data_retention,
            notification_timeout=args.notification_timeout,
            moderate_notifications=moderate_notifications,
            important_metadata_keys=important_metadata_keys,
            wait_before_notification_timeout=args.wait_before_notification_timeout,
            overwrite=(not args.no_overwrite),
        )

        if project is None:
            if not args.silent:
                for error in errors:
                    print(error, file=sys.stderr)
            return False
        else:
            if not args.silent:
                print('Project saved: %s' % project.url)
            return True



#!/usr/bin/env python3

import jinja2
import sys


sys.path.append('..')


from squad_client.core.api import SquadApi
from squad_client.core.models import Squad


# Configure SQUAD url
SquadApi.configure(url='https://qa-reports.linaro.org/')

# Generate a report with all groups
groups = Squad().groups()

templateLoader = jinja2.FileSystemLoader(searchpath="./")
templateEnv = jinja2.Environment(loader=templateLoader)
TEMPLATE_FILE = "example_template.html"
template = templateEnv.get_template(TEMPLATE_FILE)
outputText = template.render(groups=groups.values())
with open('generated_report.html', 'w') as reportFile:
    reportFile.write(outputText)



import requests
import requests_cache
import urllib
import re

from squad_client import logging
from squad_client.version import __min_squad_version__ as min_squad_version


url_validator_regex = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


logger = logging.getLogger(__name__)


# ref: https://git.linaro.org/lava/lava-lab.git/tree/shared/lab-scripts/rerun_health_check.py#n13
class NullAuth(requests.auth.AuthBase):
    '''force requests to ignore the ``.netrc``

    Some sites do not support regular authentication, but we still
    want to store credentials in the ``.netrc`` file and submit them
    as form elements. Without this, requests would otherwise use the
    .netrc which leads, on some sites, to a 401 error.

    Use with::

        requests.get(url, auth=NullAuth())

    Copied from: https://github.com/psf/requests/issues/2773#issuecomment-174312831
    '''

    def __call__(self, r):
        return r


class ApiException(requests.exceptions.RequestException):
    pass


class SquadApi:
    url = None
    token = None
    headers = None
    version = None

    @staticmethod
    def configure(url, token=None, cache=0):
        if url is None or url_validator_regex.match(url) is None:
            raise ApiException('Malformed url: "%s"' % url)

        if token:
            SquadApi.token = token
            SquadApi.headers = {"Authorization": 'token %s' % token}

        SquadApi.url = url if url[-1] == '/' else url + '/'
        logger.debug('SquadApi: url = "%s" and token = "%s"' % (SquadApi.url, 'yes' if SquadApi.token else 'no'))

        squad_server_version = SquadApi.get('/api/version/')
        if squad_server_version.status_code == 404:
            logger.warning('Could not identify squad server version!')
        else:
            SquadApi.version = squad_server_version.text
            if squad_server_version.text.split('.') < min_squad_version.split('.'):
                logger.warning('You are running squad-client against and old (< %s) version of squad server, somethings might not work as expected!' % min_squad_version)

        if cache > 0:
            logger.debug('Caching results in "squad_client_cache.sqlite" for %d seconds' % cache)
            requests_cache.install_cache('squad_client_cache', expire_after=cache)

    @staticmethod
    def get(endpoint, params={}):
        return SquadApi.__request__('GET', endpoint, params=params)

    @staticmethod
    def post(endpoint, params={}, data={}):
        return SquadApi.__request__('POST', endpoint, params=params, data=data)

    @staticmethod
    def patch(endpoint, params={}, data={}):
        return SquadApi.__request__('PATCH', endpoint, params=params, data=data)

    @staticmethod
    def delete(endpoint, params={}, data={}):
        return SquadApi.__request__('DELETE', endpoint, params=params, data=data)

    @staticmethod
    def __request__(method, endpoint, **kwargs):
        if SquadApi.url is None:
            raise ApiException('Missing "url" in SquadApi configuration. Example: `export SQUAD_HOST=http://qa-reports.linaro.org`')

        if endpoint.startswith('http'):

            parsed_url = urllib.parse.urlparse(endpoint)
            tmp_url = '%s://%s/' % (parsed_url.scheme, parsed_url.netloc)
            if SquadApi.url != tmp_url:
                raise ApiException('Given url (%s) is does not match pre-configured one!' % tmp_url)

            endpoint = parsed_url.path

            params = kwargs.get('params', {})
            params.update(urllib.parse.parse_qs(parsed_url.query))
            kwargs['params'] = params

        url = '%s%s' % (SquadApi.url, endpoint if endpoint[0] != '/' else endpoint[1:])
        logger.debug('%s %s' % (method, url))

        if SquadApi.headers:
            kwargs['headers'] = SquadApi.headers

        try:
            response = requests.request(method, url, auth=NullAuth(), **kwargs)
            if response.status_code == 401:
                msg = 'Unauthorized access to "%s"' % url
                # logger.error(msg)
                if SquadApi.token is None:
                    raise ApiException('%s. Consider `export SQUAD_TOKEN=your-squad-token`' % msg)
            elif response.status_code == 403:
                raise ApiException('%s %s is forbidden' % (method, endpoint))
            elif response.status_code == 500:
                logger.error('You hit a bug in SQUAD, please report it at https://github.com/Linaro/squad/issues/new so we can get it fixed.')

            return response

        except requests.exceptions.ConnectionError as e:
            raise ApiException('Error Connecting: %s' % e)
        except requests.exceptions.Timeout as e:
            raise ApiException('Timeout Error: %s' % e)
        except requests.exceptions.RequestException as e:
            raise ApiException('OOps: Something unexpected happened while requesting the API: %s' % e)



import json
import re


def first(_dict):
    if _dict is None or len(_dict) == 0:
        return None
    return next(iter(_dict.values()))


def parse_test_name(name):
    suite_name, test_name = name.split('/', 1)
    return (suite_name, test_name)


def parse_metric_name(name):
    return parse_test_name(name)


def split_group_project_slug(group_project_slug):
    group_slug, project_slug = group_project_slug.split('/')
    return (group_slug, project_slug)


def split_build_url(build_slug):
    group_slug, project_slug, _, build_version = build_slug.split('/')
    return (group_slug, project_slug, build_version)


def to_json(thing):
    return json.dumps(thing)


def get_class_name(obj):
    return obj.__class__.__name__


def getid(url):
    matches = re.search(r'^.*/(\d+)/$', url)
    try:
        _id = int(matches.group(1))
        return _id
    except ValueError:
        return -1



import json
import os
import yaml

from squad_client import logging
from squad_client.shortcuts import submit_results
from squad_client.core.command import SquadClientCommand


logger = logging.getLogger(__name__)


class SubmitCommand(SquadClientCommand):
    command = "submit"
    help_text = "submit results to SQUAD"

    def register(self, subparser):
        parser = super(SubmitCommand, self).register(subparser)
        result_group = parser.add_mutually_exclusive_group()
        result_group.add_argument(
            "--results",
            help="File with test results to submit. Max 5MB. JSON and YAML formats are supported",
        )
        result_group.add_argument(
            "--result-name",
            help="Single result name. Please use suite_name/test_name as value for this parameter"
        )
        parser.add_argument(
            "--result-value",
            help="Single result output (pass/fail/skip)",
            choices=["pass", "fail", "skip"],
        )
        parser.add_argument(
            "--metrics",
            help="File with metrics(benchmarsk) to submit. Max 5MB. JSON and YAML formats are supported",
        )
        parser.add_argument(
            "--metadata",
            help="File with metadata to submit. Max 5MB. JSON and YAML formats are supported",
        )
        parser.add_argument(
            "--attachments",
            nargs="+",
            help="Job attachments. Multiple files are allowed",
            default=[],
        )
        parser.add_argument("--logs", help="Test log file path")
        parser.add_argument(
            "--group", help="SQUAD group where results are stored", required=True
        )
        parser.add_argument(
            "--project", help="SQUAD project where results are stored", required=True
        )
        parser.add_argument(
            "--build", help="Build version where results are stored", required=True
        )
        parser.add_argument(
            "--environment",
            help="Build environent where results are stored",
            required=True,
        )

    def __check_file(self, file_path):
        if not os.path.exists(file_path):
            logger.error("Requested file %s doesn't exist" % file_path)
            return False
        # check file size and quit if the file is too big
        if os.stat(file_path).st_size > 5242881:
            logger.error("%s - file too big" % file_path)
            return False
        return True

    def __read_input_file(self, file_path):
        if not self.__check_file(file_path):
            return None

        _, ext = os.path.splitext(file_path)
        if ext not in ['.json', '.yml', '.yaml']:
            logger.error('File "%s" does not have a JSON or YAML file extension' % file_path)
            return None

        parser = json.load if ext == '.json' else yaml.safe_load
        parser_exception = json.JSONDecodeError if ext == '.json' else yaml.YAMLError
        output_dict = None
        with open(file_path, "r") as results_file:
            try:
                output_dict = parser(results_file)
            except parser_exception as e:
                logger.error('Failed parsing file "%s": %s' % (file_path, e))

        return output_dict

    def run(self, args):
        results_dict = {}
        metrics_dict = {}
        metadata_dict = None
        logs_file = None
        if args.result_name:
            if not args.result_value:
                logger.error("Test result value is required")
                return False
            results_dict = {args.result_name: args.result_value}

        if args.results:
            results_dict = self.__read_input_file(args.results)

            if results_dict is None:
                return False

        if args.metrics:
            metrics_dict = self.__read_input_file(args.metrics)
            if metrics_dict is None:
                return False

        if args.result_name is None and args.results is None and args.metrics is None:
            logger.error(
                "At least one of --result-name, --results, --metrics is required"
            )
            return False

        if args.metadata:
            metadata_dict = self.__read_input_file(args.metadata)
            if metadata_dict is None:
                return False

        if args.logs:
            if not self.__check_file(args.logs):
                return False
            with open(args.logs, "r") as logs_file_source:
                logs_file = logs_file_source.read()

        for filename in args.attachments:
            if not self.__check_file(filename):
                return False

        if results_dict:
            # check dictionary correctness
            for key, value in iter(results_dict.items()):
                if type(key) is not str:
                    logger.error("Non-string key detected")
                    return False
                if type(value) not in [str, dict]:
                    logger.error("Incompatible results detected")
                    return False

        if metrics_dict:
            # check dictionary correctness
            for key, value in iter(metrics_dict.items()):
                if type(key) is not str:
                    logger.error("Non-string key detected")
                    return False
                if type(value) not in [float, int, list]:
                    logger.error("Incompatible metrics detected")
                    return False

        if metadata_dict:
            # check dictionary correctness
            for key, value in iter(metadata_dict.items()):
                if type(key) is not str:
                    logger.error("Non-string key detected")
                    return False
                if type(value) not in [str, dict, int, list]:
                    logger.error("Incompatible metadata detected")
                    return False

        submit_results(
            group_project_slug="%s/%s" % (args.group, args.project),
            build_version=args.build,
            env_slug=args.environment,
            tests=results_dict,
            metrics=metrics_dict,
            log=logs_file,
            metadata=metadata_dict,
        )

        return True



import os


from squad_client.settings import *  # noqa


tests_dir = os.path.dirname(os.path.abspath(__file__))


DEFAULT_SQUAD_PORT = 9000
DEFAULT_SQUAD_DATABASE_NAME = os.path.join(tests_dir, 'squad.sqlite3')
DEFAULT_SQUAD_DATABASE_CONFIG = 'ENGINE=django.db.backends.sqlite3:NAME=%s' % DEFAULT_SQUAD_DATABASE_NAME



from collections import defaultdict

from .core.models import ALL, Squad, Group, Project, Build, Environment, Test, Metric, TestRun, SquadObjectException
from .utils import split_build_url, first, split_group_project_slug, getid


squad = Squad()


def compare_builds(baseline_id, build_id):
    return Project.compare_builds(baseline_id, build_id)


def retrieve_latest_builds(project_full_name, count=10):
    return squad.builds(count=count, project__full_name=project_full_name)


def retrieve_build_results(build_url):
    group_slug, project_slug, build_version = split_build_url(build_url)
    group = squad.group(group_slug)
    project = group.project(project_slug)
    environments = project.environments(count=ALL)
    suites = project.suites(count=ALL)
    build = project.build(build_version)

    if not build:
        return None

    results = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(dict))))

    tests = build.tests(fields='id,short_name,status,suite,environment').values()
    for test in tests:
        env = environments[getid(test.environment)]
        suite = suites[getid(test.suite)]
        results[env]['tests'][suite][test.short_name] = test.status

    metrics = squad.metrics(test_run__build_id=build.id, fields='id,short_name,result,suite,test_run').values()
    testruns = build.testruns(fields='id,environment')
    for metric in metrics:
        testrun = testruns[getid(metric.test_run)]
        env = environments[getid(testrun.environment)]
        suite = suites[getid(metric.suite)]
        results[env]['metrics'][suite][metric.short_name] = metric.results

    return results


def submit_results(group_project_slug=None, build_version=None, env_slug=None, tests={}, metrics={}, log=None, metadata=None, attachments=None):
    group_slug, project_slug = split_group_project_slug(group_project_slug)

    # TODO: validate input

    group = Group()
    project = Project()
    build = Build()
    testrun = TestRun()
    environment = Environment()

    group.slug = group_slug
    project.group = group
    project.slug = project_slug
    environment.slug = env_slug
    environment.project = project
    build.project = project
    build.version = build_version

    testrun.log = log
    testrun.build = build
    testrun.metadata = metadata
    testrun.attachments = attachments
    testrun.environment = environment

    for test_name in tests.keys():
        test = Test()
        test.name = test_name
        testrun.add_test(test)

        if type(tests[test_name]) is dict:
            test.status = tests[test_name].get('result')  # raise error if result is invalid
            test.log = tests[test_name].get('log')
        else:
            test.status = tests[test_name]

    for metric_name in metrics.keys():
        metric = Metric()
        metric.name = metric_name
        metric.result = metrics[metric_name]
        testrun.add_metric(metric)

    return testrun.submit_results()


def create_or_update_project(group_slug=None, slug=None, name=None, description=None, settings=None,
                             is_public=None, html_mail=None, moderate_notifications=None, is_archived=None,
                             email_template=None, plugins=None, important_metadata_keys=None,
                             wait_before_notification_timeout=None, notification_timeout=None,
                             data_retention=None, overwrite=False):
    errors = []
    group = None
    project = None

    if group_slug is None:
        errors.append('Group slug is required')
        return None, errors

    group = first(squad.groups(slug=group_slug))
    if group is None:
        errors.append('Group "%s" not found' % group_slug)
        return None, errors

    if slug is None:
        errors.append('Project slug is required')
        return None, errors

    project = group.project(slug)
    if project is not None:
        if not overwrite:
            errors.append('Project exists already')
            return None, errors
    else:
        project = Project()
        project.group = group
        project.slug = slug

    if name:
        project.name = name
    if plugins:
        project.enabled_plugins_list = plugins
    if settings:
        project.project_settings = settings
    if html_mail is not None:
        project.html_mail = html_mail
    if is_public is not None:
        project.is_public = is_public
    if is_archived is not None:
        project.is_archived = is_archived
    if description:
        project.description = description
    if data_retention is not None:
        project.data_retention_days = data_retention
    if notification_timeout is not None:
        project.notification_timeout = notification_timeout
    if moderate_notifications is not None:
        project.moderate_notifications = moderate_notifications
    if important_metadata_keys:
        project.important_metadata_keys = '\n'.join(important_metadata_keys) if type(important_metadata_keys) == list else important_metadata_keys
    if wait_before_notification_timeout is not None:
        project.wait_before_notification = wait_before_notification_timeout

    try:
        project.save()
    except SquadObjectException as e:
        errors.append(str(e))

    if len(errors):
        return None, errors
    return project, []



from squad_client.core.api import SquadApi
from squad_client.core.models import Squad


SquadApi.configure(url='http://localhost:8000/')
groups = Squad().groups()
for _id in groups:
    print(groups[_id].slug)



import IPython
import sys

from squad_client import logging
from squad_client.core.command import SquadClientCommand


logger = logging.getLogger(__name__)


class ShellCommand(SquadClientCommand):
    command = 'shell'
    help_text = 'run squad-client on shell'

    def register(self, subparser):
        parser = super(ShellCommand, self).register(subparser)
        parser.add_argument('script', nargs='?', help='python script to run')
        parser.add_argument('--script-params', dest='script_params', help='script parameters')

    def run(self, args):
        if args.script:
            with open(args.script, 'r') as script_source:
                try:
                    program = compile(script_source.read(), args.script, 'exec')
                except SyntaxError as e:
                    logger.error('Cannot run "%s": %s' % (args.script, e))
                    return False
            sys.argv = [args.script] + args.script_params.split(" ")
            exec(program)
            return True
        else:
            IPython.embed()
            return True



from squad_client import logging
from squad_client.core.command import SquadClientCommand


INCLUDE_TESTS_CMD = True


logger = logging.getLogger(__name__)


try:
    import tests
except ImportError:
    INCLUDE_TESTS_CMD = False


class TestCommand(SquadClientCommand):
    command = 'test'
    help_text = 'test squad_client code'

    def register(self, subparser):
        if not INCLUDE_TESTS_CMD:
            return False
        parser = super(TestCommand, self).register(subparser)
        parser.add_argument('tests', nargs='*', help='list of tests to run')
        parser.add_argument('-v', '--verbose', help='verbose mode', action='store_true', default=False)
        parser.add_argument('--coverage', help='run tests with coverage', action='store_true', default=False)

    def run(self, args):
        logger.info('Running tests')
        return tests.run(args.coverage, args.tests, args.verbose)



from unittest import TestCase


from . import settings
from squad_client.core.api import SquadApi
from squad_client.core.models import Squad
from squad_client.utils import first
from squad_client.shortcuts import (
    retrieve_latest_builds,
    retrieve_build_results,
    submit_results,
    create_or_update_project,
)


class ShortcutsTest(TestCase):
    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(url="http://localhost:%s" % settings.DEFAULT_SQUAD_PORT)

    def test_retrieve_latest_builds(self):
        builds = retrieve_latest_builds("my_group/my_project", count=5)
        self.assertEqual(5, len(builds))

    def test_retrieve_build_results(self):
        results = retrieve_build_results("my_group/my_project/build/my_build")
        self.assertIsNotNone(results)


class SubmitResultsShortcutTest(TestCase):
    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(
            url="http://localhost:%s" % settings.DEFAULT_SQUAD_PORT,
            token="193cd8bb41ab9217714515954e8724f651ef8601",
        )

    def test_basic(self):
        metadata = {"job_id": "12345", "a-metadata-field": "value"}
        tests = {"testa": "pass", "testb": {"result": "pass", "log": "the log"}}
        metrics = {"metrica": 42}
        success = submit_results(
            group_project_slug="my_group/my_project",
            build_version="my_build",
            env_slug="my_env",
            tests=tests,
            metrics=metrics,
            metadata=metadata,
        )

        results = self.squad.tests(name="testa")
        self.assertTrue(len(results) > 0)
        self.assertTrue(success)

    def test_malformed_data(self):
        # job_id already exists
        metadata = {"job_id": "12345", "a-metadata-field": "value"}
        tests = {
            "test-malformed": "pass",
            "testb": {"result": "pass", "log": "the log"},
        }
        metrics = {"metrica": 42}
        success = submit_results(
            group_project_slug="my_group/my_project",
            build_version="my_build",
            env_slug="my_env",
            tests=tests,
            metrics=metrics,
            metadata=metadata,
        )

        results = self.squad.tests(name="test-malformed")
        self.assertTrue(len(results) == 0)
        self.assertFalse(success)


class CreateOrUpdateShortcutTest(TestCase):
    def setUp(self):
        self.squad = Squad()
        SquadApi.configure(
            url="http://localhost:%s" % settings.DEFAULT_SQUAD_PORT,
            token="193cd8bb41ab9217714515954e8724f651ef8601",
        )

        self.group_slug = 'my_group'

    def assertEqualProjects(self, project1, project2):
        self.assertEqual(project1.id, project2.id)
        self.assertEqual(project1.name, project2.name)
        self.assertEqual(project1.description, project2.description)
        self.assertEqual(project1.is_public, project2.is_public)
        self.assertEqual(project1.html_mail, project2.html_mail)
        self.assertEqual(project1.moderate_notifications, project2.moderate_notifications)
        self.assertEqual(project1.is_archived, project2.is_archived)
        self.assertEqual(project1.enabled_plugins_list, project2.enabled_plugins_list)
        self.assertEqual(project1.important_metadata_keys, project2.important_metadata_keys)
        self.assertEqual(project1.wait_before_notification, project2.wait_before_notification)
        self.assertEqual(project1.notification_timeout, project2.notification_timeout)
        self.assertEqual(project1.data_retention_days, project2.data_retention_days)

    def test_minimum_parameters(self):
        project_slug = 'project-with-minimum-parameteres'
        project, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
        )

        self.assertIsNotNone(project)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqual(check_project.id, project.id)

        project.delete()

    def test_all_parameters(self):
        project_slug = 'project-with-all-parameteres'
        project, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='project name',
            description='project description',
            settings='{"SETTING_KEY": "SETTING VALUE"}',
            is_public=True,
            html_mail=False,
            moderate_notifications=False,
            is_archived=False,
            plugins=['linux-log-parser'],
            important_metadata_keys="important-key-1,important key 2",
            wait_before_notification_timeout=60,
            notification_timeout=120,
            data_retention=1,
        )

        self.assertIsNotNone(project)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqualProjects(check_project, project)

        project.delete()

    def test_overwrite(self):
        project_slug = 'project-with-overwritten-data'
        project, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name',
        )

        self.assertIsNotNone(project)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqual(check_project.id, project.id)
        self.assertEqual(check_project.name, project.name)

        project_edited, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name edited',
            overwrite=True,
        )

        self.assertIsNotNone(project_edited)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqual(check_project.id, project_edited.id)
        self.assertEqual(check_project.name, project_edited.name)

        project_edited.delete()

    def test_overwrite_selected_fields_only(self):
        project_slug = 'project-with-overwritten-data-specific-fields-only'
        description = 'project description'
        settings = '{"SETTING_KEY": "SETTING VALUE"}'
        is_public = True
        html_mail = False
        moderate_notifications = False
        is_archived = False
        plugins = ['linux-log-parser']
        important_metadata_keys = 'important-key-1,important key 2'
        wait_before_notification_timeout = 60
        notification_timeout = 120
        data_retention = 1

        project, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name',
            description=description,
            settings=settings,
            is_public=is_public,
            html_mail=html_mail,
            moderate_notifications=moderate_notifications,
            is_archived=is_archived,
            plugins=plugins,
            important_metadata_keys=important_metadata_keys,
            wait_before_notification_timeout=wait_before_notification_timeout,
            notification_timeout=notification_timeout,
            data_retention=data_retention,
        )

        self.assertIsNotNone(project)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqualProjects(check_project, project)

        project_edited, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name edited',
            overwrite=True,
        )

        self.assertIsNotNone(project_edited)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqualProjects(check_project, project_edited)
        self.assertEqual(description, project_edited.description)
        self.assertEqual(is_public, project_edited.is_public)
        self.assertEqual(html_mail, project_edited.html_mail)
        self.assertEqual(moderate_notifications, project_edited.moderate_notifications)
        self.assertEqual(is_archived, project_edited.is_archived)
        self.assertEqual(plugins, project_edited.enabled_plugins_list)
        self.assertEqual(important_metadata_keys, project_edited.important_metadata_keys)
        self.assertEqual(wait_before_notification_timeout, project_edited.wait_before_notification)
        self.assertEqual(notification_timeout, project_edited.notification_timeout)
        self.assertEqual(data_retention, project_edited.data_retention_days)

        project_edited.delete()

    def test_no_overwrite(self):
        project_slug = 'project-without-overwritten-data'
        project, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name',
        )

        self.assertIsNotNone(project)
        self.assertEqual(0, len(errors))

        check_project = first(self.squad.projects(group__slug=self.group_slug, slug=project_slug))
        self.assertEqual(check_project.id, project.id)
        self.assertEqual(check_project.name, project.name)

        project_edited, errors = create_or_update_project(
            group_slug=self.group_slug,
            slug=project_slug,
            name='new name edited',
            overwrite=False,
        )

        self.assertIsNone(project_edited)
        self.assertEqual(1, len(errors))
        self.assertEqual(['Project exists already'], errors)

        project.delete()



import os
import sys
import subprocess as sp
import time
import requests
import socket


from . import settings
from squad_client import logging

# Possible outcomes of running squad-admin process
OK = 0  # all good, exited with 0
TIMEOUT = 1  # as name says, squad-admin timed out
ERROR = 2  # squad-admin didn't time out but returned non-zero status


base_path = os.path.dirname(__file__)
squad_settings_file_path = os.path.join(base_path, 'squad_settings.py')


class SquadAdmin:
    def __init__(self, env={'DATABASE': settings.DEFAULT_SQUAD_DATABASE_CONFIG, 'SQUAD_EXTRA_SETTINGS': squad_settings_file_path, 'SQUAD_STORAGE_DIR': '/tmp/squad_storage'}):
        self.cmd = ['squad-admin']
        self.env = os.environ.copy()
        self.env.update(env)
        self.__truncate_database__()
        self.logger = logging.getLogger('squad-admin')
        self.daemons = []

    def __del__(self):
        if len(self.daemons):
            self.logger.info('Terminating daemons')

        for daemon in self.daemons:
            if daemon.poll() is None:
                daemon.kill()

    def __truncate_database__(self):
        if os.path.isfile(settings.DEFAULT_SQUAD_DATABASE_NAME):
            os.remove(settings.DEFAULT_SQUAD_DATABASE_NAME)

    def __run_process__(self, args, timeout=10, daemon=False, input=None, stdin=sp.DEVNULL, stdout=sp.DEVNULL, stderr=sp.DEVNULL):
        proc = sp.Popen(self.cmd + args, env=self.env, stdin=stdin, stdout=stdout, stderr=stderr)
        proc.ok = False

        if not daemon:
            try:
                proc.out, proc.err = proc.communicate(input=input, timeout=timeout)
                proc.ok = (proc.returncode == 0)
            except sp.TimeoutExpired:
                self.logger.error('Running "%s" time out after %i seconds!' % (' '.join(self.cmd + args), timeout))
                proc.kill()
                proc.out, proc.err = proc.communicate()
        else:
            self.daemons.append(proc)

        return proc

    def migrate(self):
        proc = self.__run_process__(['migrate'], stderr=sp.PIPE)
        if not proc.ok:
            self.logger.error('Failed to migrate!')
            print(proc.err.decode('utf-8'), file=sys.stderr)
        return proc

    def runserver(self, port=8000):
        # --noreload forces single threaded server. Ref: https://docs.djangoproject.com/en/dev/ref/django-admin/#cmdoption-runserver-noreload
        proc = self.__run_process__(['runserver', '--noreload', str(port)], daemon=True)

        attempts = 5
        while attempts > 0:
            attempts -= 1
            try:
                response = requests.get('http://localhost:%s' % str(port))
                if response.ok:
                    self.logger.debug('`squad-admin runserver` has started successfully!')
                    proc.ok = True
            except requests.exceptions.ConnectionError:
                self.logger.debug('Checking if `squad-admin runserver` is running... attempt %s' % str(attempts))
                time.sleep(1)

        if not proc.ok:
            self.logger.error('Failed to start `squad-admin runserver`!')
        return proc

    def shell(self, input=None):
        proc = self.__run_process__(['shell'], input=input, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.PIPE)
        return proc


class SquadService:

    def __init__(self, port=settings.DEFAULT_SQUAD_PORT):
        self.port = port
        self.host = 'http://localhost:%s' % str(self.port)
        self.service = None
        self.squad_admin = SquadAdmin()
        self.logger = logging.getLogger('squad-service')

    def __del__(self):
        self.stop()

    def __port_in_use__(self):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            return s.connect_ex(('localhost', self.port)) == 0

    def start(self):
        self.logger.info('Starting a fresh squad instance on port %s' % self.port)

        if self.__port_in_use__():
            self.logger.error('... port already in use!')
            return False

        self.logger.info('Creating database schema')
        proc = self.squad_admin.migrate()
        if not proc.ok:
            return False

        self.service = self.squad_admin.runserver(port=self.port)
        if not self.service.ok:
            self.logger.error('Failed to start squad service "%s"' % self.service.stderr.read().decode())
            return False

        return True

    def is_running(self):
        return self.service is not None and self.service.poll() is None

    def apply_fixtures(self, fixtures_path):
        self.logger.info('Applying %s' % fixtures_path)

        with open(fixtures_path, 'rb') as f:
            fixtures_content = f.read()

        proc = self.squad_admin.shell(fixtures_content)
        if not proc.ok:
            error_message = proc.out + proc.err
            self.logger.error(error_message.decode())

        return proc.ok

    def stop(self):
        if self.is_running():
            self.logger.debug('Terminating Squad')
            self.service.kill()
            self.service.wait(timeout=5)



import unittest

from . import settings
from squad_client.core.api import SquadApi
from squad_client.core.models import Squad, ALL, Project
from squad_client.utils import first


SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT)


class SquadTest(unittest.TestCase):

    def setUp(self):
        self.squad = Squad()

    def test_groups(self):
        groups = self.squad.groups()
        self.assertTrue(True, len(groups))

    def test_not_found_groups(self):
        groups = self.squad.groups(name__startswith='no group with this name')
        self.assertEqual(0, len(groups))

    def test_groups_with_count(self):
        all_groups = self.squad.groups(count=ALL)
        self.assertEqual(2, len(all_groups))

        one_groups = self.squad.groups(count=1)
        self.assertEqual(1, len(one_groups))

    def test_not_found_group(self):
        not_found_group = self.squad.group('this-group-does-not-really-exist')
        self.assertEqual(None, not_found_group)

    def test_group(self):
        group = self.squad.group('my_group')
        self.assertTrue(group is not None)

    def test_projects(self):
        projects = self.squad.projects()
        self.assertTrue(True, len(projects))

    def test_builds(self):
        builds = self.squad.builds()
        self.assertTrue(True, len(builds))

    def test_testjobs(self):
        testjobs = self.squad.testjobs()
        self.assertTrue(True, len(testjobs))

    def test_testruns(self):
        testruns = self.squad.testruns()
        self.assertTrue(True, len(testruns))

    def test_tests(self):
        tests = self.squad.tests()
        self.assertTrue(True, len(tests))

    def test_suites(self):
        suites = self.squad.suites()
        self.assertTrue(True, len(suites))

    def test_environments(self):
        environments = self.squad.environments()
        self.assertTrue(True, len(environments))

    def test_backends(self):
        backends = self.squad.backends()
        self.assertTrue(True, len(backends))

    def test_emailtemplates(self):
        emailtemplates = self.squad.emailtemplates()
        self.assertTrue(True, len(emailtemplates))

    def test_knownissues(self):
        knownissues = self.squad.knownissues()
        self.assertTrue(True, len(knownissues))

    def test_suitemetadata(self):
        suitemetadata = self.squad.suitemetadata()
        self.assertTrue(True, len(suitemetadata))

    def test_annotations(self):
        annotations = self.squad.annotations()
        self.assertTrue(True, len(annotations))

    def test_metricthresholds(self):
        metricthresholds = self.squad.metricthresholds()
        self.assertTrue(True, len(metricthresholds))

    def test_reports(self):
        reports = self.squad.reports()
        self.assertTrue(True, len(reports))


class BuildTest(unittest.TestCase):

    def setUp(self):
        self.build = first(Squad().builds(version='my_build'))

    def test_basic(self):
        self.assertTrue(self.build is not None)

    def test_build_metadata(self):
        metadata = self.build.metadata
        self.assertTrue(metadata.__id__ != '')

    def test_build_tests(self):
        tests = self.build.tests().values()
        self.assertEqual(4, len(tests))

    def test_build_tests_per_environment(self):
        tests = self.build.tests(environment__slug='my_env').values()
        self.assertEqual(4, len(tests))

    def test_build_tests_per_environment_not_found(self):
        tests = self.build.tests(environment__slug='mynonexistentenv').values()
        self.assertEqual(0, len(tests))

    def test_build_tests_change_cache_on_different_filters(self):
        tests = self.build.tests(environment__slug='my_env').values()
        self.assertEqual(4, len(tests))

        tests = self.build.tests(environment__slug='mynonexistentenv').values()
        self.assertEqual(0, len(tests))


class TestRunTest(unittest.TestCase):

    def setUp(self):
        self.testrun = first(Squad().testruns(count=1))

    def test_basic(self):
        self.assertTrue(self.testrun is not None)

    def test_testrun_status(self):
        status = self.testrun.summary()
        self.assertEqual(1, status.tests_fail)


class ProjectTest(unittest.TestCase):

    def setUp(self):
        SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT, token='193cd8bb41ab9217714515954e8724f651ef8601')

        self.project = first(Squad().projects(slug='my_project'))
        self.build = first(Squad().builds(version='my_build'))
        self.build2 = first(Squad().builds(version='my_build2'))

    def test_basic(self):
        self.assertTrue(self.project is not None)

    def test_project_environments(self):
        environments = self.project.environments()
        self.assertEqual(2, len(environments))

        environment = self.project.environment('my_env')
        self.assertEqual(environment.slug, 'my_env')

    def test_project_suites(self):
        suites = self.project.suites()
        self.assertEqual(2, len(suites))

        suite = self.project.suite('my_suite')
        self.assertEqual(suite.slug, 'my_suite')

    def test_compare_builds_from_same_project(self):
        comparison = self.project.compare_builds(self.build2.id, self.build.id)
        self.assertEqual('Cannot report regressions/fixes on a non-finished builds', comparison[0])

    def test_compare_builds_from_same_project_force(self):
        comparison = self.project.compare_builds(self.build2.id, self.build.id, force=True)
        self.assertEqual({}, comparison['regressions'])
        self.assertEqual({}, comparison['fixes'])

    def test_create_project(self):
        group = Squad().group('my_group')
        slug = 'test-create-project'
        new_project = Project()
        new_project.slug = slug
        new_project.group = group
        new_project.enabled_plugins_list = ['linux-log-parser']
        new_project.save()

        check_project = first(Squad().projects(slug=slug, group__slug=group.slug))
        self.assertEqual(new_project.id, check_project.id)

        new_project.delete()

    def test_save_project_settings(self):
        settings = 'SETTING: value'
        self.project.project_settings = settings
        self.project.save()
        project = first(Squad().projects(slug=self.project.slug))
        self.assertTrue(project is not None)


class GroupTest(unittest.TestCase):

    def setUp(self):
        SquadApi.configure(url='http://localhost:%s' % settings.DEFAULT_SQUAD_PORT, token='193cd8bb41ab9217714515954e8724f651ef8601')

        self.group = first(Squad().groups(slug='my_group'))

    def test_create_project(self):
        project_slug = 'test-create-project2'
        self.group.create_project(slug=project_slug)

        check_project = Squad().projects(slug=project_slug, group__slug=self.group.slug)
        self.assertEqual(1, len(check_project))

        p = first(check_project)
        p.delete()


class SuiteTest(unittest.TestCase):

    def setUp(self):
        self.suite = first(Squad().suites(slug='my_suite'))

    def test_basic(self):
        self.assertTrue(self.suite is not None)

    def test_suite_tests(self):
        tests = self.suite.tests()
        self.assertEqual(4, len(tests))
